//! Entry points

use alloc::{string::String, vec};

use casper_types::{
    CLType, CLTyped, EntryPoint, EntryPointAccess, EntryPointType, EntryPoints, Key, Parameter,
    U256,
};

use crate::constants::{
    AMOUNT, ASSIGN_WORK_ENTRY_POINT_NAME, GET_ASSIGNED_WORK_ENTRY_POINT_NAME,
    STAKE_AVAILABILITY_ENTRY_POINT_NAME, WORKER, WORK_CONTRACT,
};

// Entry point: Stake availability toward one or more work contracts
pub fn stake_availability() -> EntryPoint {
    EntryPoint::new(
        String::from(STAKE_AVAILABILITY_ENTRY_POINT_NAME),
        vec![Parameter::new(AMOUNT, U256::cl_type())],
        CLType::Unit,
        EntryPointAccess::Public,
        EntryPointType::Contract,
    )
}

// Entry point: Assign work to an available worker
pub fn assign_work() -> EntryPoint {
    EntryPoint::new(
        String::from(ASSIGN_WORK_ENTRY_POINT_NAME),
        vec![Parameter::new(WORK_CONTRACT, Key::cl_type())],
        CLType::Unit,
        EntryPointAccess::Public,
        EntryPointType::Contract,
    )
}

// Entry point: Get assigned work
pub fn get_assigned_work() -> EntryPoint {
    EntryPoint::new(
        String::from(GET_ASSIGNED_WORK_ENTRY_POINT_NAME),
        vec![Parameter::new(WORKER, Key::cl_type())],
        Key::cl_type(),
        EntryPointAccess::Public,
        EntryPointType::Contract,
    )
}

pub fn generate_entry_points() -> EntryPoints {
    let mut entry_points = EntryPoints::new();
    entry_points.add_entry_point(stake_availability());
    entry_points.add_entry_point(assign_work());
    entry_points.add_entry_point(get_assigned_work());
    entry_points
}
